#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

const long BUFFER_SIZE = 128;
const long TYPES_COUNT = 2;
const long VEHICLES_COUNT = 4;
const long MAX_VERTICES = 110; //100

const long INFINITY = 1000000000;
const double eps = 1e-5;

template<class T>T Min(T a, T b){return a < b ? a : b;}

struct SVehicle
{
	long mType;
	long mSpeed;
	long mFirstCost;
	long mFirstDist;
	long mRestCost;
	long mWaitTime;

	double CalcTime(long dist)
	{
		//if(dist < mFirstDist)dist = mFirstDist;
		return mWaitTime + (double)dist * 60 / mSpeed;
	}

	long CalcCost(long dist)
	{
		if(dist < mFirstDist)dist = mFirstDist;
		long cost = mFirstCost;
		dist -= mFirstDist;
		cost += dist * mRestCost;
		return cost;
	}
};

SVehicle vehicles[VEHICLES_COUNT] =
{
	{0, 10, 5, 1, 2, 2},
	{1, 30, 20, 2, 10, 3},
	{1, 50, 20, 2, 16, 10},
	{1, 40, 2, 5, 1, 30}
};

struct SVehicleWay
{
	long mCost;
	double mTime;

	void Print(void)
	{
		if(mCost >= INFINITY)puts("UNREACHABLE");
		else printf("%ld %.2lf\n", mCost, mTime);
	}
};

struct SEdge
{
	SVehicleWay mVehicles[VEHICLES_COUNT];
};

struct STypeGraph
{
	long mDists[MAX_VERTICES][MAX_VERTICES];
	long mcVertices;

	void Init(long cVertices)
	{
		mcVertices = cVertices;
		long iBegin, iEnd;
		for(iBegin = 0; iBegin < cVertices; ++iBegin)
			for(iEnd = 0; iEnd < cVertices; ++iEnd)
				mDists[iBegin][iEnd] = INFINITY;
		for(iBegin = 0; iBegin < cVertices; ++iBegin)
			mDists[iBegin][iBegin] = 0;
	}

	void AddEdge(long iBegin, long iEnd, long distance)
	{
		mDists[iBegin][iEnd] = Min(mDists[iBegin][iEnd], distance);
	}

	void DoTransitiveClosure(void)
	{
		long iBegin, iEnd, iCenter;
		for(iCenter = 0; iCenter < mcVertices; ++iCenter)
			for(iBegin = 0; iBegin < mcVertices; ++iBegin)
				for(iEnd = 0; iEnd < mcVertices; ++iEnd)
					if(mDists[iBegin][iEnd] > mDists[iBegin][iCenter] + mDists[iCenter][iEnd])
						mDists[iBegin][iEnd] = mDists[iBegin][iCenter] + mDists[iCenter][iEnd];
	}

	int HasEdge(long iBegin, long iEnd)const
	{
		return mDists[iBegin][iEnd] < INFINITY;
	}
};

struct SCostTimeGraph
{
	SEdge mWays[MAX_VERTICES][MAX_VERTICES];
	long mcVertices;
	SVehicleWay mFastests[MAX_VERTICES];
	SVehicleWay mCheapests[MAX_VERTICES];

	void Initialize(long cVertices)
	{
		mcVertices = cVertices;
		long iBegin, iEnd, iVehicle;
		for(iBegin = 0; iBegin < cVertices; ++iBegin)
			for(iEnd = 0; iEnd < cVertices; ++iEnd)
				for(iVehicle = 0; iVehicle < VEHICLES_COUNT; ++iVehicle)
				{
					mWays[iBegin][iEnd].mVehicles[iVehicle].mCost = INFINITY;
					mWays[iBegin][iEnd].mVehicles[iVehicle].mTime = INFINITY;
				}
	}

	void BuildCostsTimes(const STypeGraph &type1, const STypeGraph &type2)
	{
		const STypeGraph* typeGraphs[2] = {&type1, &type2};

		long iBegin, iEnd, iVehicle;
		for(iBegin = 0; iBegin < mcVertices; ++iBegin)
			for(iEnd = 0; iEnd < mcVertices; ++iEnd)
				for(iVehicle = 0; iVehicle < VEHICLES_COUNT; ++iVehicle)
					if(typeGraphs[vehicles[iVehicle].mType]->HasEdge(iBegin, iEnd))
					{
						long dist = typeGraphs[vehicles[iVehicle].mType]->mDists[iBegin][iEnd];
						mWays[iBegin][iEnd].mVehicles[iVehicle].mCost = Min<long>(mWays[iBegin][iEnd].mVehicles[iVehicle].mCost, vehicles[iVehicle].CalcCost(dist));
						mWays[iBegin][iEnd].mVehicles[iVehicle].mTime = Min<double>(mWays[iBegin][iEnd].mVehicles[iVehicle].mTime, vehicles[iVehicle].CalcTime(dist));
					}
	}

	void RelaxFastests(long iBegin, long iEnd, long iVehicle)
	{
		if(fabs(mFastests[iEnd].mTime - (mFastests[iBegin].mTime + mWays[iBegin][iEnd].mVehicles[iVehicle].mTime)) < eps)
		{
			if(mFastests[iEnd].mCost > mFastests[iBegin].mCost + mWays[iBegin][iEnd].mVehicles[iVehicle].mCost)
				mFastests[iEnd].mCost = mFastests[iBegin].mCost + mWays[iBegin][iEnd].mVehicles[iVehicle].mCost;
		}
		else if(mFastests[iEnd].mTime > mFastests[iBegin].mTime + mWays[iBegin][iEnd].mVehicles[iVehicle].mTime)
		{
			mFastests[iEnd].mTime = mFastests[iBegin].mTime + mWays[iBegin][iEnd].mVehicles[iVehicle].mTime;
			mFastests[iEnd].mCost = mFastests[iBegin].mCost + mWays[iBegin][iEnd].mVehicles[iVehicle].mCost;
		}
	}

	void RelaxCheapests(long iBegin, long iEnd, long iVehicle)
	{
		if(mCheapests[iEnd].mCost == mCheapests[iBegin].mCost + mWays[iBegin][iEnd].mVehicles[iVehicle].mCost)
		{
			if(mCheapests[iEnd].mTime > mCheapests[iBegin].mTime + mWays[iBegin][iEnd].mVehicles[iVehicle].mTime)
				mCheapests[iEnd].mTime = mCheapests[iBegin].mTime + mWays[iBegin][iEnd].mVehicles[iVehicle].mTime;
		}
		else if(mCheapests[iEnd].mCost > mCheapests[iBegin].mCost + mWays[iBegin][iEnd].mVehicles[iVehicle].mCost)
		{
			mCheapests[iEnd].mCost = mCheapests[iBegin].mCost + mWays[iBegin][iEnd].mVehicles[iVehicle].mCost;
			mCheapests[iEnd].mTime = mCheapests[iBegin].mTime + mWays[iBegin][iEnd].mVehicles[iVehicle].mTime;
		}
	}

	void BuildPaths(long iStart)
	{
		long iVertice;
		for(iVertice = 0; iVertice < mcVertices; ++iVertice)
		{
			mFastests[iVertice].mCost = INFINITY;
			mFastests[iVertice].mTime = INFINITY;
			mCheapests[iVertice].mCost = INFINITY;
			mCheapests[iVertice].mTime = INFINITY;
		}

		mFastests[iStart].mCost = 0;
		mFastests[iStart].mTime = 0;
		mCheapests[iStart].mCost = 0;
		mCheapests[iStart].mTime = 0;

		long iBegin, iEnd, iVehicle;
		for(iVertice = 0; iVertice < mcVertices; ++iVertice)
			for(iBegin = 0; iBegin < mcVertices; ++iBegin)
				for(iEnd = 0; iEnd < mcVertices; ++iEnd)
					for(iVehicle = 0; iVehicle < VEHICLES_COUNT; ++iVehicle)
					{
						RelaxCheapests(iBegin, iEnd, iVehicle);
						RelaxFastests(iBegin, iEnd, iVehicle);
					}
	}

};

struct SCity
{
	STypeGraph mTypeGraphs[TYPES_COUNT];
	SCostTimeGraph mCTs;
	long mcVertices, mBegin, mEnd;
	SVehicleWay mCheapest;
	SVehicleWay mFastest;

	int Read(void)
	{
		long cEdges;
		if(scanf("%ld %ld", &mcVertices, &cEdges) != 2)return 0;
		scanf("%ld %ld", &mBegin, &mEnd);

		mTypeGraphs[0].Init(mcVertices);
		mTypeGraphs[1].Init(mcVertices);

		long iBegin, iEnd, iEdge, distance;
		char buffer[BUFFER_SIZE];
		for(iEdge = 0; iEdge < cEdges; ++iEdge)
		{
			scanf("%ld %ld %ld %s", &iBegin, &iEnd, &distance, buffer);
			if(buffer[0] == 'N' || buffer[0] == 'A')
			{
				mTypeGraphs[0].AddEdge(iBegin, iEnd, distance);
				mTypeGraphs[0].AddEdge(iEnd, iBegin, distance);
			}
			if(buffer[0] == 'M' || buffer[0] == 'A')
			{
				mTypeGraphs[1].AddEdge(iBegin, iEnd, distance);
				mTypeGraphs[1].AddEdge(iEnd, iBegin, distance);
			}
		}

		return 1;
	}

	void Calculate(void)
	{
		mTypeGraphs[0].DoTransitiveClosure();
		mTypeGraphs[1].DoTransitiveClosure();

		mCTs.Initialize(mcVertices);
		mCTs.BuildCostsTimes(mTypeGraphs[0], mTypeGraphs[1]);
		mCTs.BuildPaths(mBegin);

		mCheapest = mCTs.mCheapests[mEnd];
		mFastest = mCTs.mFastests[mEnd];
	}

	void WriteAnswer(long iCase)
	{
		printf("Case#%ld\n", iCase);
		mFastest.Print();
		mCheapest.Print();
	}
};

int main(void)
{
#ifndef ONLINE_JUDGE
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
#endif

	SCity Dhaka;
	long iCase = 0;
	while(Dhaka.Read())
	{
		Dhaka.Calculate();
		Dhaka.WriteAnswer(++iCase);
	}

#ifndef ONLINE_JUDGE
	fclose(stdout);
	fclose(stdin);
#endif

	return 0;
}